@@ -296,15 +296,13 @@ class StreamMsgType(enum.Enum):
     TRADE = 22
     OPEN_INTEREST = 23
 
-    STOP = 30
+    # Tape commands
+    START = 30
+    RESTART = 31
+    STOP = 32
 
-    # Experimental
-    REQUEST_SERVER_LIST = 300
-    REQUEST_OPTIMAL_SERVER = 301
-    OPTIMAL_SERVER = 302
-    PACKET = 303
-    BAN_IP = 304
-    POPULATION = 305
+    # Misc
+    REQ_RESPONSE = 40
 
     @classmethod
     def from_code(cls, code: int) -> StreamMsgType:
@@ -530,44 +528,44 @@ class TradeCondition(enum.Enum):
     TRADE_THRU_EXEMPT = 108
     IMPLIED = 109
     OTC = 110
-    TBA_111 = 111
-    TBA_112 = 112
-    TBA_113 = 113
-    TBA_114 = 114
-    TBA_115 = 115
-    TBA_116 = 116
-    TBA_117 = 117
-    TBA_118 = 118
-    TBA_119 = 119
-    TBA_120 = 120
-    TBA_121 = 121
-    TBA_122 = 122
-    TBA_123 = 123
-    TBA_124 = 124
-    TBA_125 = 125
-    TBA_126 = 126
-    TBA_127 = 127
-    TBA_128 = -128
-    TBA_129 = -127
-    TBA_130 = -126
-    TBA_131 = -125
-    TBA_132 = -124
-    TBA_133 = -123
-    TBA_134 = -122
-    TBA_135 = -121
-    TBA_136 = -120
-    TBA_137 = -119
-    TBA_138 = -118
-    TBA_139 = -117
-    TBA_140 = -116
-    TBA_141 = -115
-    TBA_142 = -114
-    TBA_143 = -113
-    TBA_144 = -112
-    TBA_145 = -111
-    TBA_146 = -110
-    TBA_147 = -109
-    TBA_148 = -108
+    MKT_SUPERVISION = 111
+    RESERVED_77 = 112
+    RESERVED_91 = 113
+    CONTINGENT_UTP = 114
+    ODD_LOT = 115
+    RESERVED_89 = 116
+    CORRECTED_LAST = 117
+    OPRA_EXT_HOURS = 118
+    RESERVED_78 = 119
+    RESERVED_81 = 120
+    RESERVED_84 = 121
+    RESERVED_878 = 122
+    RESERVED_90 = 123
+    QUALIFIED_CONTINGENT_TRADE = 124
+    SINGLE_LEG_AUCTION_NON_ISO = 125
+    SINGLE_LEG_AUCTION_ISO = 126
+    SINGLE_LEG_CROSS_NON_ISO = 127
+    SINGLE_LEG_CROSS_ISO = 128
+    SINGLE_LEG_FLOOR_TRADE = 129
+    MULTI_LEG_AUTO_ELECTRONIC_TRADE = 130
+    MULTI_LEG_AUCTION = 131
+    MULTI_LEG_CROSS = -132
+    MULTI_LEG_FLOOR_TRADE = 133
+    MULTI_LEG_AUTO_ELEC_TRADE_AGAINST_SINGLE_LEG = 134
+    STOCK_OPTIONS_AUCTION = 135
+    MULTI_LEG_AUCTION_AGAINST_SINGLE_LEG = 136
+    MULTI_LEG_FLOOR_TRADE_AGAINST_SINGLE_LEG = 137
+    STOCK_OPTIONS_AUTO_ELEC_TRADE = 138
+    STOCK_OPTIONS_CROSS = 139
+    STOCK_OPTIONS_FLOOR_TRADE = 140
+    STOCK_OPTIONS_AUTO_ELEC_TRADE_AGAINST_SINGLE_LEG = 141
+    STOCK_OPTIONS_AUCTION_AGAINST_SINGLE_LEG = 142
+    STOCK_OPTIONS_FLOOR_TRADE_AGAINST_SINGLE_LEG = 143
+    MULTI_LEG_FLOOR_TRADE_OF_PROPRIETARY_PRODUCTS = 144
+    BID_AGGRESSOR = 145
+    ASK_AGGRESSOR = 146
+    MULTI_LATERAL_COMPRESSION_TRADE_OF_PROPRIETARY_DATA_PRODUCTS = 147
+    EXTENDED_HOURS_TRADE = 148
     UNDEFINED = 10000
 
     @classmethod
@@ -668,3 +666,26 @@ class QuoteCondition(enum.Enum):
             if code == member.value:
                 return member
         return QuoteCondition.UNDEFINED
+
+
+@enum.unique
+class StreamResponseType(enum.Enum):
+    """Codes used to ID types of requests/responses."""
+
+    # Internal client communication
+    SUBSCRIBED = 0  # This doesn't guarantee you will get data back for the contract. It just means that if this contract exists, you will get data for it.
+    TIMED_OUT = 1  # The request to stream something timed out.
+    MAX_STREAMS_REACHED = 2  # Returned when you are streaming too many contracts.
+    INVALID_PERMS = 3  # If you do not have permissions for the stream request.
+
+    @classmethod
+    def from_code(cls, code: int) -> StreamResponseType:
+        """Create a MessageType by its associated code.
+
+        :raises EnumParseError: If the code does not match a MessageType
+        """
+        for member in cls:
+            if code == member.value:
+                return member
+        raise exceptions._EnumParseError(code, cls)
+
